# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tasks_fields.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12tasks_fields.proto\x12\x19\x61rmonik.api.grpc.v1.tasks\"R\n\x10TaskSummaryField\x12>\n\x05\x66ield\x18\x01 \x01(\x0e\x32/.armonik.api.grpc.v1.tasks.TaskSummaryEnumField\"P\n\x0fTaskOptionField\x12=\n\x05\x66ield\x18\x01 \x01(\x0e\x32..armonik.api.grpc.v1.tasks.TaskOptionEnumField\"\'\n\x16TaskOptionGenericField\x12\r\n\x05\x66ield\x18\x01 \x01(\t\"\x80\x02\n\tTaskField\x12I\n\x12task_summary_field\x18\x01 \x01(\x0b\x32+.armonik.api.grpc.v1.tasks.TaskSummaryFieldH\x00\x12G\n\x11task_option_field\x18\x02 \x01(\x0b\x32*.armonik.api.grpc.v1.tasks.TaskOptionFieldH\x00\x12V\n\x19task_option_generic_field\x18\x03 \x01(\x0b\x32\x31.armonik.api.grpc.v1.tasks.TaskOptionGenericFieldH\x00\x42\x07\n\x05\x66ield*\xd9\x05\n\x14TaskSummaryEnumField\x12\'\n#TASK_SUMMARY_ENUM_FIELD_UNSPECIFIED\x10\x00\x12#\n\x1fTASK_SUMMARY_ENUM_FIELD_TASK_ID\x10\x10\x12&\n\"TASK_SUMMARY_ENUM_FIELD_SESSION_ID\x10\x01\x12(\n$TASK_SUMMARY_ENUM_FIELD_OWNER_POD_ID\x10\t\x12+\n\'TASK_SUMMARY_ENUM_FIELD_INITIAL_TASK_ID\x10\n\x12\"\n\x1eTASK_SUMMARY_ENUM_FIELD_STATUS\x10\x02\x12&\n\"TASK_SUMMARY_ENUM_FIELD_CREATED_AT\x10\x03\x12(\n$TASK_SUMMARY_ENUM_FIELD_SUBMITTED_AT\x10\x0b\x12&\n\"TASK_SUMMARY_ENUM_FIELD_STARTED_AT\x10\x04\x12$\n TASK_SUMMARY_ENUM_FIELD_ENDED_AT\x10\x05\x12\x34\n0TASK_SUMMARY_ENUM_FIELD_CREATION_TO_END_DURATION\x10\x06\x12\x36\n2TASK_SUMMARY_ENUM_FIELD_PROCESSING_TO_END_DURATION\x10\x07\x12#\n\x1fTASK_SUMMARY_ENUM_FIELD_POD_TTL\x10\x0c\x12(\n$TASK_SUMMARY_ENUM_FIELD_POD_HOSTNAME\x10\r\x12\'\n#TASK_SUMMARY_ENUM_FIELD_RECEIVED_AT\x10\x0e\x12\'\n#TASK_SUMMARY_ENUM_FIELD_ACQUIRED_AT\x10\x0f\x12!\n\x1dTASK_SUMMARY_ENUM_FIELD_ERROR\x10\x08*\xc3\x03\n\x13TaskOptionEnumField\x12&\n\"TASK_OPTION_ENUM_FIELD_UNSPECIFIED\x10\x00\x12\'\n#TASK_OPTION_ENUM_FIELD_MAX_DURATION\x10\x01\x12&\n\"TASK_OPTION_ENUM_FIELD_MAX_RETRIES\x10\x02\x12#\n\x1fTASK_OPTION_ENUM_FIELD_PRIORITY\x10\x03\x12\'\n#TASK_OPTION_ENUM_FIELD_PARTITION_ID\x10\x04\x12+\n\'TASK_OPTION_ENUM_FIELD_APPLICATION_NAME\x10\x05\x12.\n*TASK_OPTION_ENUM_FIELD_APPLICATION_VERSION\x10\x06\x12\x30\n,TASK_OPTION_ENUM_FIELD_APPLICATION_NAMESPACE\x10\x07\x12.\n*TASK_OPTION_ENUM_FIELD_APPLICATION_SERVICE\x10\x08\x12&\n\"TASK_OPTION_ENUM_FIELD_ENGINE_TYPE\x10\tB\x1c\xaa\x02\x19\x41rmoniK.Api.gRPC.V1.Tasksb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tasks_fields_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\252\002\031ArmoniK.Api.gRPC.V1.Tasks'
  _globals['_TASKSUMMARYENUMFIELD']._serialized_start=516
  _globals['_TASKSUMMARYENUMFIELD']._serialized_end=1245
  _globals['_TASKOPTIONENUMFIELD']._serialized_start=1248
  _globals['_TASKOPTIONENUMFIELD']._serialized_end=1699
  _globals['_TASKSUMMARYFIELD']._serialized_start=49
  _globals['_TASKSUMMARYFIELD']._serialized_end=131
  _globals['_TASKOPTIONFIELD']._serialized_start=133
  _globals['_TASKOPTIONFIELD']._serialized_end=213
  _globals['_TASKOPTIONGENERICFIELD']._serialized_start=215
  _globals['_TASKOPTIONGENERICFIELD']._serialized_end=254
  _globals['_TASKFIELD']._serialized_start=257
  _globals['_TASKFIELD']._serialized_end=513
# @@protoc_insertion_point(module_scope)



from __future__ import annotations
from grpc import Channel
from typing import cast, Tuple, List

from ..protogen.client.sessions_service_pb2_grpc import SessionsStub
from ..protogen.common.submitter_common_pb2 import SessionFilter
from ..protogen.common.sessions_common_pb2 import GetSessionRequest, GetSessionResponse, ListSessionsRequest, ListSessionsResponse, SessionRaw, CancelSessionRequest, CancelSessionResponse
from ..protogen.common.sessions_filters_pb2 import Filters as rawFilters, FiltersAnd as rawFilterAnd, FilterField as rawFilterField, FilterStatus as rawFilterStatus
from ..protogen.common.sessions_fields_pb2 import *
from ..common.filter import StringFilter, StatusFilter, DateFilter, NumberFilter, Filter
from ..protogen.common.sort_direction_pb2 import SortDirection
from ..common import Direction, Session
from ..protogen.common.sessions_fields_pb2 import SessionField, SessionRawField, SESSION_RAW_ENUM_FIELD_STATUS, TaskOptionGenericField

class SessionFieldFilter:
    """
    Enumeration of the available filters
    """
    STATUS = StatusFilter(SessionField(session_raw_field=SessionRawField(field=SESSION_RAW_ENUM_FIELD_STATUS)), rawFilters, rawFilterAnd, rawFilterField, rawFilterStatus)

    @staticmethod
    def task_options_key(option_key: str) -> StringFilter:
        """
        Filter for the TaskOptions.Options dictionary
        Args:
            option_key: key in the dictionary

        Returns:
            Corresponding filter
        """
        return StringFilter(SessionField(task_option_generic_field=TaskOptionGenericField(field=option_key)), rawFilters, rawFilterAnd, rawFilterField)

class ArmoniKSessions:

    def __init__(self, grpc_channel: Channel):
        """ Session service client

        Args:
            grpc_channel: gRPC channel to use
        """
        self._client = SessionsStub(grpc_channel)

    def list_sessions(self, task_filter: Filter, page: int = 0, page_size: int = 1000, sort_field: Filter = SessionFieldFilter.STATUS, sort_direction: SortDirection = Direction.ASC) -> Tuple[int, List[Session]]:
        """
        List sessions

        Args:
            session_filter (Filter): Filter to apply when listing sessions.
            page: page number to request, this can be useful when paginating the result, defaults to 0
            page_size: size of a page, defaults to 1000
            sort_field: field on which to sort the resulting list, defaults to the status
            sort_direction: direction of the sort, defaults to ascending

        Returns:
            A tuple containing :
            - The total number of sessions for the given filter
            - The obtained list of sessions
        """
        request : ListSessionsRequest = ListSessionsRequest(
            page=page,
            page_size=page_size,
            filters=cast(rawFilters, task_filter.to_disjunction().to_message()),
            sort=ListSessionsRequest.Sort(field=cast(SessionField, sort_field.field), direction=sort_direction),
        )
        list_response : ListSessionsResponse = self._client.ListSessions(request)
        return list_response.total, [Session.from_message(t) for t in list_response.sessions]

    def cancel_session(self, session_id: str) -> None:
        """Cancel a session

        Args:
            session_id: Id of the session to b cancelled
        """
        self._client.CancelSession(CancelSessionRequest(session_id=session_id))
        


# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from ..common import agent_common_pb2 as agent__common__pb2


class AgentStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.CreateTask = channel.stream_unary(
                '/armonik.api.grpc.v1.agent.Agent/CreateTask',
                request_serializer=agent__common__pb2.CreateTaskRequest.SerializeToString,
                response_deserializer=agent__common__pb2.CreateTaskReply.FromString,
                )
        self.CreateResultsMetaData = channel.unary_unary(
                '/armonik.api.grpc.v1.agent.Agent/CreateResultsMetaData',
                request_serializer=agent__common__pb2.CreateResultsMetaDataRequest.SerializeToString,
                response_deserializer=agent__common__pb2.CreateResultsMetaDataResponse.FromString,
                )
        self.CreateResults = channel.unary_unary(
                '/armonik.api.grpc.v1.agent.Agent/CreateResults',
                request_serializer=agent__common__pb2.CreateResultsRequest.SerializeToString,
                response_deserializer=agent__common__pb2.CreateResultsResponse.FromString,
                )
        self.NotifyResultData = channel.unary_unary(
                '/armonik.api.grpc.v1.agent.Agent/NotifyResultData',
                request_serializer=agent__common__pb2.NotifyResultDataRequest.SerializeToString,
                response_deserializer=agent__common__pb2.NotifyResultDataResponse.FromString,
                )
        self.SubmitTasks = channel.unary_unary(
                '/armonik.api.grpc.v1.agent.Agent/SubmitTasks',
                request_serializer=agent__common__pb2.SubmitTasksRequest.SerializeToString,
                response_deserializer=agent__common__pb2.SubmitTasksResponse.FromString,
                )
        self.GetResourceData = channel.unary_unary(
                '/armonik.api.grpc.v1.agent.Agent/GetResourceData',
                request_serializer=agent__common__pb2.DataRequest.SerializeToString,
                response_deserializer=agent__common__pb2.DataResponse.FromString,
                )
        self.GetCommonData = channel.unary_unary(
                '/armonik.api.grpc.v1.agent.Agent/GetCommonData',
                request_serializer=agent__common__pb2.DataRequest.SerializeToString,
                response_deserializer=agent__common__pb2.DataResponse.FromString,
                )
        self.GetDirectData = channel.unary_unary(
                '/armonik.api.grpc.v1.agent.Agent/GetDirectData',
                request_serializer=agent__common__pb2.DataRequest.SerializeToString,
                response_deserializer=agent__common__pb2.DataResponse.FromString,
                )


class AgentServicer(object):
    """Missing associated documentation comment in .proto file."""

    def CreateTask(self, request_iterator, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateResultsMetaData(self, request, context):
        """*
        Create the metadata of multiple results at once
        Data have to be uploaded separately
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateResults(self, request, context):
        """*
        Create one result with data included in the request
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def NotifyResultData(self, request, context):
        """*
        Notify Agent that a data file representing the Result to upload is available in the shared folder
        The name of the file should be the result id
        Blocks until data are stored in Object Storage
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SubmitTasks(self, request, context):
        """*
        Create tasks metadata and submit task for processing.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetResourceData(self, request, context):
        """*
        Retrieve Resource Data from the Agent
        Data is stored in the shared folder between Agent and Worker as a file with the result id as name
        Blocks until data are available in the shared folder
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetCommonData(self, request, context):
        """*
        Retrieve Resource Data from the Agent
        Data is stored in the shared folder between Agent and Worker as a file with the result id as name
        Blocks until data are available in the shared folder
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetDirectData(self, request, context):
        """*
        Retrieve Resource Data from the Agent
        Data is stored in the shared folder between Agent and Worker as a file with the result id as name
        Blocks until data are available in the shared folder
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_AgentServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'CreateTask': grpc.stream_unary_rpc_method_handler(
                    servicer.CreateTask,
                    request_deserializer=agent__common__pb2.CreateTaskRequest.FromString,
                    response_serializer=agent__common__pb2.CreateTaskReply.SerializeToString,
            ),
            'CreateResultsMetaData': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateResultsMetaData,
                    request_deserializer=agent__common__pb2.CreateResultsMetaDataRequest.FromString,
                    response_serializer=agent__common__pb2.CreateResultsMetaDataResponse.SerializeToString,
            ),
            'CreateResults': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateResults,
                    request_deserializer=agent__common__pb2.CreateResultsRequest.FromString,
                    response_serializer=agent__common__pb2.CreateResultsResponse.SerializeToString,
            ),
            'NotifyResultData': grpc.unary_unary_rpc_method_handler(
                    servicer.NotifyResultData,
                    request_deserializer=agent__common__pb2.NotifyResultDataRequest.FromString,
                    response_serializer=agent__common__pb2.NotifyResultDataResponse.SerializeToString,
            ),
            'SubmitTasks': grpc.unary_unary_rpc_method_handler(
                    servicer.SubmitTasks,
                    request_deserializer=agent__common__pb2.SubmitTasksRequest.FromString,
                    response_serializer=agent__common__pb2.SubmitTasksResponse.SerializeToString,
            ),
            'GetResourceData': grpc.unary_unary_rpc_method_handler(
                    servicer.GetResourceData,
                    request_deserializer=agent__common__pb2.DataRequest.FromString,
                    response_serializer=agent__common__pb2.DataResponse.SerializeToString,
            ),
            'GetCommonData': grpc.unary_unary_rpc_method_handler(
                    servicer.GetCommonData,
                    request_deserializer=agent__common__pb2.DataRequest.FromString,
                    response_serializer=agent__common__pb2.DataResponse.SerializeToString,
            ),
            'GetDirectData': grpc.unary_unary_rpc_method_handler(
                    servicer.GetDirectData,
                    request_deserializer=agent__common__pb2.DataRequest.FromString,
                    response_serializer=agent__common__pb2.DataResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'armonik.api.grpc.v1.agent.Agent', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Agent(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def CreateTask(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_unary(request_iterator, target, '/armonik.api.grpc.v1.agent.Agent/CreateTask',
            agent__common__pb2.CreateTaskRequest.SerializeToString,
            agent__common__pb2.CreateTaskReply.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateResultsMetaData(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/armonik.api.grpc.v1.agent.Agent/CreateResultsMetaData',
            agent__common__pb2.CreateResultsMetaDataRequest.SerializeToString,
            agent__common__pb2.CreateResultsMetaDataResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateResults(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/armonik.api.grpc.v1.agent.Agent/CreateResults',
            agent__common__pb2.CreateResultsRequest.SerializeToString,
            agent__common__pb2.CreateResultsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def NotifyResultData(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/armonik.api.grpc.v1.agent.Agent/NotifyResultData',
            agent__common__pb2.NotifyResultDataRequest.SerializeToString,
            agent__common__pb2.NotifyResultDataResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SubmitTasks(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/armonik.api.grpc.v1.agent.Agent/SubmitTasks',
            agent__common__pb2.SubmitTasksRequest.SerializeToString,
            agent__common__pb2.SubmitTasksResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetResourceData(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/armonik.api.grpc.v1.agent.Agent/GetResourceData',
            agent__common__pb2.DataRequest.SerializeToString,
            agent__common__pb2.DataResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetCommonData(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/armonik.api.grpc.v1.agent.Agent/GetCommonData',
            agent__common__pb2.DataRequest.SerializeToString,
            agent__common__pb2.DataResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetDirectData(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/armonik.api.grpc.v1.agent.Agent/GetDirectData',
            agent__common__pb2.DataRequest.SerializeToString,
            agent__common__pb2.DataResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



from . import objects_pb2 as _objects_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class ProcessRequest(_message.Message):
    __slots__ = ["communication_token", "session_id", "task_id", "task_options", "expected_output_keys", "payload_id", "data_dependencies", "data_folder", "configuration"]
    COMMUNICATION_TOKEN_FIELD_NUMBER: _ClassVar[int]
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    TASK_ID_FIELD_NUMBER: _ClassVar[int]
    TASK_OPTIONS_FIELD_NUMBER: _ClassVar[int]
    EXPECTED_OUTPUT_KEYS_FIELD_NUMBER: _ClassVar[int]
    PAYLOAD_ID_FIELD_NUMBER: _ClassVar[int]
    DATA_DEPENDENCIES_FIELD_NUMBER: _ClassVar[int]
    DATA_FOLDER_FIELD_NUMBER: _ClassVar[int]
    CONFIGURATION_FIELD_NUMBER: _ClassVar[int]
    communication_token: str
    session_id: str
    task_id: str
    task_options: _objects_pb2.TaskOptions
    expected_output_keys: _containers.RepeatedScalarFieldContainer[str]
    payload_id: str
    data_dependencies: _containers.RepeatedScalarFieldContainer[str]
    data_folder: str
    configuration: _objects_pb2.Configuration
    def __init__(self, communication_token: _Optional[str] = ..., session_id: _Optional[str] = ..., task_id: _Optional[str] = ..., task_options: _Optional[_Union[_objects_pb2.TaskOptions, _Mapping]] = ..., expected_output_keys: _Optional[_Iterable[str]] = ..., payload_id: _Optional[str] = ..., data_dependencies: _Optional[_Iterable[str]] = ..., data_folder: _Optional[str] = ..., configuration: _Optional[_Union[_objects_pb2.Configuration, _Mapping]] = ...) -> None: ...

class ProcessReply(_message.Message):
    __slots__ = ["output"]
    OUTPUT_FIELD_NUMBER: _ClassVar[int]
    output: _objects_pb2.Output
    def __init__(self, output: _Optional[_Union[_objects_pb2.Output, _Mapping]] = ...) -> None: ...

class HealthCheckReply(_message.Message):
    __slots__ = ["status"]
    class ServingStatus(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        UNKNOWN: _ClassVar[HealthCheckReply.ServingStatus]
        SERVING: _ClassVar[HealthCheckReply.ServingStatus]
        NOT_SERVING: _ClassVar[HealthCheckReply.ServingStatus]
    UNKNOWN: HealthCheckReply.ServingStatus
    SERVING: HealthCheckReply.ServingStatus
    NOT_SERVING: HealthCheckReply.ServingStatus
    STATUS_FIELD_NUMBER: _ClassVar[int]
    status: HealthCheckReply.ServingStatus
    def __init__(self, status: _Optional[_Union[HealthCheckReply.ServingStatus, str]] = ...) -> None: ...



from __future__ import annotations
import os
from typing import Optional, Dict, List, Tuple, Union, cast

from ..common import TaskOptions, TaskDefinition, Task
from ..protogen.common.agent_common_pb2 import CreateTaskRequest, CreateResultsMetaDataRequest, CreateResultsMetaDataResponse, NotifyResultDataRequest
from ..protogen.common.objects_pb2 import TaskRequest, DataChunk, InitTaskRequest, TaskRequestHeader, Configuration
from ..protogen.worker.agent_service_pb2_grpc import AgentStub
from ..protogen.common.worker_common_pb2 import ProcessRequest


class TaskHandler:
    def __init__(self, request: ProcessRequest, agent_client: AgentStub):
        self._client: AgentStub = agent_client
        self.session_id: str = request.session_id
        self.task_id: str = request.task_id
        self.task_options: TaskOptions = TaskOptions.from_message(request.task_options)
        self.token: str = request.communication_token
        self.expected_results: List[str] = list(request.expected_output_keys)
        self.configuration: Configuration = request.configuration
        self.payload_id: str = request.payload_id
        self.data_folder: str = request.data_folder

        # TODO: Lazy load
        with open(os.path.join(self.data_folder, self.payload_id), "rb") as f:
            self.payload = f.read()

        # TODO: Lazy load
        self.data_dependencies: Dict[str, bytes] = {}
        for dd in request.data_dependencies:
            with open(os.path.join(self.data_folder, dd), "rb") as f:
                self.data_dependencies[dd] = f.read()

    def create_tasks(self, tasks: List[TaskDefinition], task_options: Optional[TaskOptions] = None) -> Tuple[List[Task], List[str]]:
        """Create new tasks for ArmoniK

        Args:
            tasks: List of task definitions
            task_options: Task Options used for this batch of tasks

        Returns:
            Tuple containing the list of successfully sent tasks, and
            the list of submission errors if any
        """
        task_requests = []

        for t in tasks:
            task_request = TaskRequest()
            task_request.expected_output_keys.extend(t.expected_output_ids)
            task_request.data_dependencies.extend(t.data_dependencies)
            task_request.payload = t.payload
            task_requests.append(task_request)
        assert self.configuration is not None
        create_tasks_reply = self._client.CreateTask(_to_request_stream(task_requests, self.token, task_options.to_message() if task_options is not None else None, self.configuration.data_chunk_max_size))
        ret = create_tasks_reply.WhichOneof("Response")
        if ret is None or ret == "error":
            raise Exception(f'Issue with server when submitting tasks : {create_tasks_reply.error}')
        elif ret == "creation_status_list":
            tasks_created = []
            tasks_creation_failed = []
            for creation_status in create_tasks_reply.creation_status_list.creation_statuses:
                if creation_status.WhichOneof("Status") == "task_info":
                    tasks_created.append(Task(id=creation_status.task_info.task_id, session_id=self.session_id, expected_output_ids=[k for k in creation_status.task_info.expected_output_keys], data_dependencies=[k for k in creation_status.task_info.data_dependencies]))
                else:
                    tasks_creation_failed.append(creation_status.error)
        else:
            raise Exception("Unknown value")
        return tasks_created, tasks_creation_failed

    def send_result(self, key: str, data: Union[bytes, bytearray]) -> None:
        """ Send task result

        Args:
            key: Result key
            data: Result data
        """
        with open(os.path.join(self.data_folder, key), "wb") as f:
            f.write(data)

        self._client.NotifyResultData(NotifyResultDataRequest(ids=[NotifyResultDataRequest.ResultIdentifier(session_id=self.session_id, result_id=key)], communication_token=self.token))

    def get_results_ids(self, names: List[str]) -> Dict[str, str]:
        return {r.name: r.result_id for r in cast(CreateResultsMetaDataResponse, self._client.CreateResultsMetaData(CreateResultsMetaDataRequest(results=[CreateResultsMetaDataRequest.ResultCreate(name=n) for n in names], session_id=self.session_id, communication_token=self.token))).results}


def _to_request_stream_internal(request, communication_token, is_last, chunk_max_size):
    req = CreateTaskRequest(
        init_task=InitTaskRequest(
            header=TaskRequestHeader(
                data_dependencies=request.data_dependencies,
                expected_output_keys=request.expected_output_keys
            )
        ),
        communication_token=communication_token
    )
    yield req
    start = 0
    payload_length = len(request.payload)
    if payload_length == 0:
        req = CreateTaskRequest(
            task_payload=DataChunk(data=b''),
            communication_token=communication_token
        )
        yield req
    while start < payload_length:
        chunk_size = min(chunk_max_size, payload_length - start)
        req = CreateTaskRequest(
            task_payload=DataChunk(data=request.payload[start:start + chunk_size]),
            communication_token=communication_token
        )
        yield req
        start += chunk_size
    req = CreateTaskRequest(
        task_payload=DataChunk(data_complete=True),
        communication_token=communication_token
    )
    yield req

    if is_last:
        req = CreateTaskRequest(
            init_task=InitTaskRequest(last_task=True),
            communication_token=communication_token
        )
        yield req


def _to_request_stream(requests, communication_token, t_options, chunk_max_size):
    if t_options is None:
        req = CreateTaskRequest(
            init_request=CreateTaskRequest.InitRequest(),
            communication_token=communication_token)
    else:
        req = CreateTaskRequest(
            init_request=CreateTaskRequest.InitRequest(task_options=t_options),
            communication_token=communication_token)
    yield req
    if len(requests) == 0:
        return
    for r in requests[:-1]:
        for req in _to_request_stream_internal(r, communication_token, False, chunk_max_size):
            yield req
    for req in _to_request_stream_internal(requests[-1], communication_token, True, chunk_max_size):
        yield req



from . import applications_fields_pb2 as _applications_fields_pb2
from . import filters_common_pb2 as _filters_common_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class FilterField(_message.Message):
    __slots__ = ["field", "filter_string"]
    FIELD_FIELD_NUMBER: _ClassVar[int]
    FILTER_STRING_FIELD_NUMBER: _ClassVar[int]
    field: _applications_fields_pb2.ApplicationField
    filter_string: _filters_common_pb2.FilterString
    def __init__(self, field: _Optional[_Union[_applications_fields_pb2.ApplicationField, _Mapping]] = ..., filter_string: _Optional[_Union[_filters_common_pb2.FilterString, _Mapping]] = ...) -> None: ...

class FiltersAnd(_message.Message):
    __slots__ = []
    AND_FIELD_NUMBER: _ClassVar[int]
    def __init__(self, **kwargs) -> None: ...

class Filters(_message.Message):
    __slots__ = []
    OR_FIELD_NUMBER: _ClassVar[int]
    def __init__(self, **kwargs) -> None: ...
